@@ -6,19 +6,16 @@ use warnings;
 use utf8;
 use version;
 
-use Carp;
-use Mail::RFC822::Address;
-use DateTime;
-use DateTime::Format::DateParse;
 use List::MoreUtils                 qw(any firstval);
 
 use Mojo::URL;
 use Mojo::JSON;
 use Mojo::DOM;
+use Mojo::Loader;
 
 use Mojolicious::Plugin::Vparam::Address;
 
-our $VERSION    = '1.18';
+our $VERSION    = '1.19';
 
 # Shift for convert ASCII char position to simple sequence 0,1,2...9,A,B,C,,,
 our $CHAR_SHIFT = ord('A') - 10;
@@ -58,7 +55,7 @@ Many predefined types
 
 =item *
 
-Shortcats for the most common uses
+Shortcuts for the most common uses
 
 =item *
 
@@ -80,7 +77,7 @@ L<Mojo::JSON::Pointer>.
 =item *
 
 Simple XML values extraction and validation using CSS selector engine
-from L<Mojo::DOM::CSS>.
+from L<Mojo::DOM::CSS> or XPath from L<XML::LibXML>.
 
 =item *
 
@@ -108,9 +105,12 @@ Just set parameter type. But if you want sub or regexp you can do it too.
     $self->plugin('Vparam');
 
     # Use in controller
-    $login = $self->vparam(login    => 'str');
-    $passw = $self->vparam(password => 'password', size => [8, 100]);
-    $email = $self->vparam(email    => 'email', optional => 1);
+    $login      = $self->vparam(login    => 'str');
+    $passw      = $self->vparam(password => 'password', size     => [8, 100]);
+    $email      = $self->vparam(email    => 'email',    optional => 1);
+    $session    = $self->vparam(session  => 'bool',     default  => 1);
+
+    $ids        = $self->vparam(ids => '@int');
 
 =head1 METHODS
 
@@ -230,6 +230,16 @@ Get parameter error string. Return 0 if no error.
     # Set error
     $self->verror('myparam', {message => 'Error message'})
 
+=head2 verrors
+
+Return erorrs count in scalar context. In list context return erorrs hash.
+
+    # List context get hash
+    my %errors = $self->verrors;
+
+    # Scalar context get count
+    die 'Errors!' if $self->verrors;
+
 =head2 vclass $name, @classes
 
 Get classes for invalid input. Return empty string if no error.
@@ -255,16 +265,6 @@ string before validation.
     # Then user submit form and you validate id. For example user submit "abc":
     # <input name="myparam" value="abc">
 
-=head2 verrors
-
-Return erorrs count in scalar context. In list context return erorrs hash.
-
-    # List context get hash
-    my %errors = $self->verrors;
-
-    # Scalar context get count
-    die 'Errors!' if $self->verrors;
-
 =head2 vtype $name, %opts
 
 Set new type $name if defined %opts. Else return type $name definition.
@@ -420,6 +420,20 @@ Enable L<Mojolicious::Validator::Validation> integration.
 
 =cut
 
+# Around deprication
+sub _load_class($) {
+    return Mojo::Loader::load_class( $_[0] ) if Mojo::Loader->can('load_class');
+    return Mojo::Loader->new->load( $_[0] )  if Mojo::Loader->can('load');
+    die 'Looks like Mojo again depricate module Mojo::Loader';
+}
+
+# Around deprication
+sub _params($$) {
+    return @{ $_[0]->every_param( $_[1] ) } if $_[0]->can('every_param');
+    return @{ $_[0]->param( $_[1] ) }       if $_[0]->can('param');
+    die 'Looks like Mojo again depricate module Mojo::Controller';
+}
+
 =head1 TYPES
 
 List of supported types:
@@ -665,8 +679,11 @@ Email adress.
 sub _check_email($) {
     return 'Value not defined'          unless defined $_[0];
     return 'Value is not set'           unless length  $_[0];
-    return 'Wrong format'
-        unless Mail::RFC822::Address::valid( $_[0] );
+
+    my $e = _load_class('Mail::RFC822::Address');
+    die $e if $e;
+
+    return 'Wrong format' unless Mail::RFC822::Address::valid( $_[0] );
     return 0;
 }
 
@@ -911,6 +928,11 @@ Result will be used as new param value.
 Parameter type. If set then some filters will be apply. See L</TYPES>.
 
     $self->vparam(myparam => 'datetime');
+    $self->vparam(myparam => {type => 'datetime'});
+    $self->vparams(
+        myparam1 => {type => 'datetime'},
+        myparam2 => {type => 'datetime'},
+    );
 
 After the application of the type used filters.
 
@@ -1004,7 +1026,7 @@ if it`s undefined by set I<skipundef>.
     );
 
     # Set all in vparams
-    $ The %params hash is empty if all parameters are not valid.
+    # The %params hash is empty if all parameters are not valid.
     %params = $self->vparams(
         -skipundef  => 1,
         param1      => 'int',
@@ -1018,7 +1040,7 @@ Arrays always return as arrayref. But undefined values will be skipped.
 
 =head2 jpath
 
-If you POST data not form but raw JSON you can user JSON Pointer selectors
+If you POST data not form but raw JSON you can use JSON Pointer selectors
 to get and validate parameters.
 
     # POST data contains:
@@ -1042,9 +1064,28 @@ Same as jpath but parse XML/HTML using CSS selectors.
     # </Point>
 
     %opts = $self->vparams(
-        address => { type => 'str', jpath => 'Point > Address' },
-        lon     => { type => 'lon', jpath => 'Point > Lon' },
-        lat     => { type => 'lat', jpath => 'Point > Lat' },
+        address => { type => 'str', cpath => 'Point > Address' },
+        lon     => { type => 'lon', cpath => 'Point > Lon' },
+        lat     => { type => 'lat', cpath => 'Point > Lat' },
+    );
+
+
+=head2 xpath
+
+Same as cpath but parse XML/HTML using XPath selectors.
+
+    # POST data contains:
+    # <Point time="2016-11-25 14:39:00 +0300">
+    #    <Address>some</Address>
+    #    <Lon>45.123456</Lon>
+    #    <Lat>38.23452</Lat>
+    # </Point>
+
+    %opts = $self->vparams(
+        address => { type => 'str',         xpath => '/Point/Address' },
+        lon     => { type => 'lon',         xpath => '/Point/Lon' },
+        lat     => { type => 'lat',         xpath => '/Point/Lat' },
+        time    => { type => 'datetime',    xpath => '/Point/@time' },
     );
 
 =cut
@@ -1153,7 +1194,7 @@ Check parameter value to be in list of defined values.
 =cut
 
 sub _in($$) {
-    confess 'Not ArrayRef'          unless 'ARRAY' eq ref $_[1];
+    die 'Not ArrayRef'              unless 'ARRAY' eq ref $_[1];
 
     return 'Value not defined'      unless defined $_[0];
     return 'Wrong value'            unless any {$_[0] eq $_} @{$_[1]};
@@ -1220,6 +1261,12 @@ sub _parse_number($) {
 sub _parse_date($;$) {
     my ($str, $tz) = @_;
 
+    my $e1 = _load_class('DateTime');
+    die $e1 if $e1;
+
+    my $e2 = _load_class('DateTime::Format::DateParse');
+    die $e2 if $e2;
+
     return undef unless defined $str;
     s{^\s+}{}, s{\s+$}{} for $str;
     return undef unless length $str;
@@ -1306,6 +1353,20 @@ sub _parse_dom($) {
     return $dom;
 }
 
+sub _parse_xml($) {
+    my $str = shift;
+    return undef unless defined $str;
+    return undef unless length  $str;
+
+    my $e = _load_class('XML::LibXML');
+    die $e if $e;
+
+    my $dom = eval{ XML::LibXML->load_xml(string => $str) };
+    warn $@ and return undef if $@;
+
+    return $dom;
+}
+
 sub _parse_phone($$$) {
     my ($str, $country, $region) = @_;
     return undef unless $str;
@@ -1599,10 +1660,7 @@ sub register {
     $app->helper(vvalue => sub{
         my ($self, $name, $default) = @_;
 
-        my @input = $self->can('every_param')
-            ? @{ $self->every_param( $name ) }
-            : $self->param( $name )
-        ;
+        my @input = _params($self, $name);
 
         my $value;
         if( not @input ) {
@@ -1704,7 +1762,7 @@ sub register {
                         $attr{ $key } = $conf->{types}{ $type }{ $key };
                     }
                 } else {
-                    confess sprintf 'Type "%s" is not defined', $type;
+                    die sprintf 'Type "%s" is not defined', $type;
                 }
             }
 
@@ -1725,12 +1783,15 @@ sub register {
                     @input = $vars->{dom}->find( $attr{cpath} )
                         ->map('text')->each;
                 }
+            } elsif( $attr{xpath} ) {
+                $vars->{xml} //= _parse_xml( $self->req->body // '' );
+                if( $vars->{xml} ) {
+                    @input = $vars->{xml}->findnodes( $attr{xpath} )
+                        ->to_literal_list;
+                }
             } else {
                 # POST parameters
-                @input = $self->can('every_param')
-                    ? @{ $self->every_param( $name ) }
-                    : $self->param( $name )
-                ;
+                @input = _params($self, $name);
             }
 
             # Set undefined value if paremeter not set
@@ -1843,8 +1904,8 @@ sub register {
     $app->helper(vparam => sub{
         my ($self, $name, $def, %attr) = @_;
 
-        confess 'Parameter name required'               unless defined $name;
-        confess 'Parameter type or definition required' unless defined $def;
+        die 'Parameter name required'               unless defined $name;
+        die 'Parameter type or definition required' unless defined $def;
 
         my $result;
 
@@ -1872,7 +1933,7 @@ sub register {
         my ($self, %attr) = @_;
 
         my $sort = delete $attr{'-sort'};
-        confess 'Key "-sort" must be ArrayRef'
+        die 'Key "-sort" must be ArrayRef'
             if defined($sort) and 'ARRAY' ne ref $sort;
 
         $attr{ $conf->{vsort_page} } = {
